@@ -90,6 +90,7 @@ static void gtk_playlist_init       (GtkPlaylist      *playlist);
 
 static void init_treeview (GtkWidget *treeview, GtkPlaylist *playlist);
 static gboolean gtk_playlist_unset_playing (GtkPlaylist *playlist);
+static void gtk_playlist_set_at_end (GtkPlaylist *playlist);
 
 GtkType
 gtk_playlist_get_type (void)
@@ -899,6 +900,13 @@ gtk_playlist_add_m3u (GtkPlaylist *playlist, const char *mrl)
 	if (eel_read_entire_file (mrl, &size, &contents) != GNOME_VFS_OK)
 		return FALSE;
 
+	if (strstr (contents, "\n") == NULL)
+	{
+		retval = gtk_playlist_add_one_mrl (playlist, contents, NULL);
+		g_free (contents);
+		return retval;
+	}
+
 	lines = g_strsplit (contents, "\n", 0);
 	g_free (contents);
 
@@ -1092,6 +1100,18 @@ gtk_playlist_add_asx (GtkPlaylist *playlist, const char *mrl)
 	return retval;
 }
 
+static gboolean
+gtk_playlist_add_ra (GtkPlaylist *playlist, const char *mrl)
+{
+	/* How nice, same format as m3u it seems */
+	return gtk_playlist_add_m3u (playlist, mrl);
+}
+#if 0 //FIXME
+static gboolean
+gtk_playlist_add_wmv (GtkPlaylist *playlist, const char *mrl)
+{
+}
+#endif
 gboolean
 gtk_playlist_add_mrl (GtkPlaylist *playlist, const char *mrl,
 		const char *display_name)
@@ -1112,6 +1132,9 @@ gtk_playlist_add_mrl (GtkPlaylist *playlist, const char *mrl,
 		return gtk_playlist_add_pls (playlist, mrl);
 	} else if (strcmp ("audio/x-ms-asx", mimetype) == 0) {
 		return gtk_playlist_add_asx (playlist, mrl);
+	} else if (strcmp ("audio/x-real-audio", mimetype) == 0
+			|| strcmp ("audio/x-pn-realaudio", mimetype) == 0) {
+		return gtk_playlist_add_ra (playlist, mrl);
 	} else if (strcmp ("x-directory/normal", mimetype) == 0) {
 		//FIXME Load all the files in the dir ?
 	}
@@ -1177,6 +1200,9 @@ gtk_playlist_has_previous_mrl (GtkPlaylist *playlist)
 	if (update_current_from_playlist (playlist) == FALSE)
 		return FALSE;
 
+	if (playlist->_priv->repeat == TRUE)
+		return TRUE;
+
 	gtk_tree_model_get_iter (playlist->_priv->model,
 			&iter,
 			playlist->_priv->current);
@@ -1193,6 +1219,7 @@ gtk_playlist_has_next_mrl (GtkPlaylist *playlist)
 
 	if (update_current_from_playlist (playlist) == FALSE)
 		return FALSE;
+
 	if (playlist->_priv->repeat == TRUE)
 		return TRUE;
 
@@ -1286,6 +1313,7 @@ void
 gtk_playlist_set_previous (GtkPlaylist *playlist)
 {
 	GtkTreeIter iter;
+	char *path;
 
 	g_return_if_fail (GTK_IS_PLAYLIST (playlist));
 	
@@ -1294,6 +1322,13 @@ gtk_playlist_set_previous (GtkPlaylist *playlist)
 
 	gtk_playlist_unset_playing (playlist);
 
+	path = gtk_tree_path_to_string (playlist->_priv->current);
+	if (strcmp (path, "0") == 0)
+	{
+		gtk_playlist_set_at_end (playlist);
+		return;
+	}
+
 	gtk_tree_model_get_iter (playlist->_priv->model,
 			&iter,
 			playlist->_priv->current);
@@ -1313,9 +1348,7 @@ gtk_playlist_set_next (GtkPlaylist *playlist)
 
 	if (gtk_playlist_has_next_mrl (playlist) == FALSE)
 	{
-		if (playlist->_priv->repeat == TRUE)
-			gtk_playlist_set_at_start (playlist);
-
+		gtk_playlist_set_at_start (playlist);
 		return;
 	}
 
@@ -1365,18 +1398,42 @@ gtk_playlist_set_at_start (GtkPlaylist *playlist)
 	update_current_from_playlist (playlist);
 }
 
+static void
+gtk_playlist_set_at_end (GtkPlaylist *playlist)
+{
+	gint nb_childs = 0;
+	g_return_if_fail (GTK_IS_PLAYLIST (playlist));
+	
+	gtk_playlist_unset_playing (playlist);
+	
+	nb_childs = gtk_tree_model_iter_n_children (playlist->_priv->model,
+			NULL);
+
+	if (playlist->_priv->current != NULL)
+	{
+		gtk_tree_path_free (playlist->_priv->current);
+		playlist->_priv->current = NULL;
+	}
+
+	if (nb_childs)
+	{
+		playlist->_priv->current = gtk_tree_path_new_from_indices
+			(nb_childs-1, -1);
+	}
+}
+
 /* This one returns a new string, in UTF8 even if the mrl is encoded
  * in the locale's encoding
  */
 gchar *
 gtk_playlist_mrl_to_title (const gchar *mrl)
 {
-	char *filename_for_display, *filename, *unescaped;
+	char *filename_for_display, *with_suffix, *filename, *unescaped;
 
 	filename = g_path_get_basename (mrl);
 	unescaped = gnome_vfs_unescape_string_for_display (filename);
 	g_free (filename);
-	filename_for_display = g_filename_to_utf8 (unescaped,
+	with_suffix = g_filename_to_utf8 (unescaped,
 			-1,             /* length */
 			NULL,           /* bytes_read */
 			NULL,           /* bytes_written */
@@ -1384,6 +1441,18 @@ gtk_playlist_mrl_to_title (const gchar *mrl)
 
 	g_free (unescaped);
 
+	if (strrchr (with_suffix, '.')
+			&& strlen (strrchr (with_suffix, '.')) < 5)
+	{
+		filename_for_display = g_strndup (with_suffix,
+				strlen (with_suffix)
+				- strlen (strrchr (with_suffix, '.')));
+	} else {
+		filename_for_display = g_strdup (with_suffix);
+	}
+
+	g_free (with_suffix);
+
 	return filename_for_display;
 }
 
@@ -1426,3 +1495,4 @@ gtk_playlist_class_init (GtkPlaylistClass *klass)
 				G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
 }
 
+
